@@ -10,8 +10,7 @@ require Exporter;
 our @ISA = qw(Exporter);
 
 our @EXPORT_OK = qw(get_dir);
-our $VERSION = '0.50';
-
+our $VERSION = '0.60';
 
 ######################################################################
 # object interface
@@ -24,6 +23,12 @@ sub new {
     my $handler = '';
     my $err_handler = '';
 
+    my $ns = {
+	      enabled => 0,
+	      uri     => 'http://gingerall.org/directory',
+	      prefix  => 'xd',
+	      };
+
     my $self = {
 		path    => $path,
 		details => $details,
@@ -31,8 +36,10 @@ sub new {
 		xml     => $xml,
 		ContentHandler => $handler,
 		ErrorHanler    => $err_handler,
-		error   => 0
+		NS      => $ns,
+		error   => 0,
 	       };
+
     bless $self, $class;
     return $self;
 }
@@ -49,7 +56,11 @@ sub parse {
     }
     if ($self->{error} == 0) {
 	eval {
-	    @xml =  get_dir($self->{path},$self->{details},$self->{depth});
+	    @xml =  get_dir($self->{path},
+			    $self->{details},
+			    $self->{depth},
+			    $self->{NS},
+			   );
 	};
     }
     if ($@) {
@@ -78,6 +89,16 @@ sub set_maxdepth {
     $self->{depth} = $depth;
 }
 
+sub enable_ns {
+    my ($self) = @_;
+    $self->{NS}->{enabled} = 1;
+}
+
+sub disable_ns {
+    my ($self) = @_;
+    $self->{NS}->{enabled} = 0;
+}
+
 sub get_arrayref {
     my $self = shift;
     return $self->{xml};
@@ -110,6 +131,11 @@ sub get_maxdepth {
     return $self->{depth};
 }
 
+sub get_ns_data {
+    my $self = shift;
+    return $self->{NS};
+}
+
 sub set_content_handler {
     my ($self, $handler) = @_;
     $handler = '' unless @_ > 1;
@@ -152,7 +178,9 @@ sub parse_SAX {
 	    $ret = _get_dir_SAX($self->{ContentHandler},
 				$self->{path},
 				$self->{details},
-				$self->{depth});
+				$self->{depth},
+				$self->{NS},
+			       );
 	};
     }
     if ($@) {
@@ -176,26 +204,37 @@ sub parse_SAX {
 # original interface
 
 sub get_dir {
-    my ($path, $details, $depth) = @_;
+    my ($path, $details, $depth, $ns) = @_;
     $details = 3 unless @_ > 1;
     $depth = 1000 unless @_ > 2;
     if ($path =~ /^(.*)[\\\/]$/) {$path = $1}
 
+    my ($pref, $decl) = _ns_data($ns);
+
     chdir ($path) or die "Path $path not found!\n";
 
     my $res = [];
     my @dirs = split("/",$path);
     my $last_dir = pop @dirs;
 
-    @$res = ('<?xml version="1.0" encoding="utf-8"?>','<dirtree>');
+    @$res = ('<?xml version="1.0" encoding="utf-8"?>',
+	     "<$pref" . 'dirtree' . "$decl>");
+    
+    if ($details>1) {
+	push @$res, "  <$pref" . "head version=\"$VERSION\">";
+	push @$res, "    <$pref" . "path>$path</$pref" . 'path>';
+	push @$res, "    <$pref" . "details>$details</$pref" . 'details>';
+	push @$res, "    <$pref" . "depth>$depth</$pref" . 'depth>';
+	push @$res, "  </$pref" . 'head>';
+    }
 
     my @stat = stat '.';
 
-    _get_current_dir($res,$details,0,$last_dir,$path,'  ',@stat);    
+    _get_current_dir($res,$details,0,$last_dir,'','  ',$ns,@stat);    
 
     # nested dirs
     if ($depth > 0) {
-	_recursive($res,1,$path,$details,$depth); 
+	_recursive($res,1,'',$details,$depth,$ns); 
     }
 
     # final dirs
@@ -203,11 +242,11 @@ sub get_dir {
 	foreach (<*>) {
 	    if (-d $_) {
 		my @dst = stat "$_";
-		my $dp;
-		if ($^O =~ /win/io) {$dp = "$path\\$_"}
-		else {$dp = "$path/$_"}
-		_get_current_dir($res,$details,1,$_,$dp,'    ',@dst);    
-		push @$res, '    </directory>';
+ 		my $dp;
+ 		if ($^O =~ /win/io) {$dp = "\\$_"}
+ 		else {$dp = "/$_"}
+		_get_current_dir($res,$details,1,$_,$dp,'    ',$ns,@dst);    
+		push @$res, "    </$pref" . 'directory>';
 	    }
 	}
     }
@@ -215,11 +254,11 @@ sub get_dir {
     # files
     foreach (<*>) {
 	if (-f $_) {
-	    _get_file($res, $_, '  ', $details);
+	    _get_file($res, $_, '  ', $details,$ns);
 	}
     }
 
-    push @$res, '  </directory>', '</dirtree>';
+    push @$res, "  </$pref" . 'directory>', "</$pref" . 'dirtree>';
     return @$res;
 }
 
@@ -231,10 +270,13 @@ sub _set_error {
     my $num = shift;
     my $msg = shift;
     my @err = ();
+
+    my ($pref, $decl) = _ns_data($self->{NS});
+
     push @err, '<?xml version="1.0" encoding="utf-8"?>';
-    push @err, '<dirtree>';
-    push @err, "<error number=\"$num\">$msg</error>";
-    push @err, '</dirtree>';
+    push @err, "<$pref" . 'dirtree' . "$decl>";
+    push @err, "<$pref" . "error number=\"$num\">$msg</$pref" . "error>";
+    push @err, "</$pref" . 'dirtree>';
     $self->{error} = $num;
     return @err;
 }
@@ -246,6 +288,9 @@ sub _recursive {
     my $dir = shift;
     my $details = shift;
     my $depth = shift;
+    my $ns = shift;
+
+    my ($pref, $decl) = _ns_data($ns);
 
     foreach my $d (<*>) {
 	if (-d $d) {
@@ -256,14 +301,14 @@ sub _recursive {
 
 	    my @stat = stat "$d";
 
-	    _get_current_dir($res,$details,$i,$d,$path,$indent,@stat);    
+	    _get_current_dir($res,$details,$i,$d,$path,$indent,$ns,@stat);    
 
 	    chdir $d;
 
 	    # nested dirs
 	    if ($depth > $i) {
 		$i++;
-		_recursive($res,$i,$path,$details,$depth); 
+		_recursive($res,$i,$path,$details,$depth,$ns); 
 		$i--; 
 	    }
 
@@ -277,8 +322,8 @@ sub _recursive {
 			my $dp;
 			if ($^O =~ /win/io) {$dp = "$path\\$_"}
 			else {$dp = "$path/$_"}
-			_get_current_dir($res,$details,$j,$_,$dp,$ind,@dst);
-			push @$res, "$ind</directory>";
+			_get_current_dir($res,$details,$j,$_,$dp,$ind,$ns,@dst);
+			push @$res, "$ind</$pref" . 'directory>';
 		    }
 		}
 	    }
@@ -286,67 +331,94 @@ sub _recursive {
 	    # files
 	    foreach (<*>) {
 		if (-f $_) {
-		    _get_file($res, $_, $indent, $details);
+		    _get_file($res, $_, $indent, $details,$ns);
 		}
 	    }
 
 	    chdir ".."; 
 
-	    push @$res, "$indent</directory>";
+	    push @$res, "$indent</$pref" . 'directory>';
 	}
     }
 }
 
 sub _get_file($$$$) {
 
-my $res     = shift;
-my $path    = shift;
-my $indent  = shift;
-my $details = shift;
+    my $res     = shift;
+    my $path    = shift;
+    my $indent  = shift;
+    my $details = shift;
+    my $ns      = shift;
 
-my @stat = stat $path;
-
-my $line = "  $indent<file name=\"$path\"";
-$line .= " uid=\"$stat[4]\" gid=\"$stat[5]\"" if $details > 2;
-$line .= '/' if $details == 1;
-$line .= '>';
-push @$res, $line;
-
-my $mode;
-if (-r $path) {$mode = 'r' }else {$mode = '-'}
-if (-w $path) {$mode .= 'w' }else {$mode .= '-'}
-if (-x $path) {$mode .= 'x' }else {$mode .= '-'}
-push @$res, "    $indent<mode code=\"$stat[2]\">$mode</mode>" 
-  if $details > 1;
-push @$res, "    $indent<size unit=\"bytes\">$stat[7]</size>" 
-  if $details > 1;
-
-my $atime = localtime($stat[8]);
-my $mtime = localtime($stat[9]);
-push @$res, "    $indent<access-time epoch=\"$stat[8]\">$atime</access-time>"
-  if $details > 2;
-push @$res, "    $indent<modify-time epoch=\"$stat[9]\">$mtime</modify-time>"
-  if $details > 1;
-
-push @$res, "  $indent</file>" if $details > 1;
+    my ($pref, $decl) = _ns_data($ns);
+    
+    my @stat = stat $path;
+    
+    my $line = "  $indent<$pref" . "file name=\"$path\"";
+    $line .= " uid=\"$stat[4]\" gid=\"$stat[5]\"" if $details > 2;
+    $line .= '/' if $details == 1;
+    $line .= '>';
+    push @$res, $line;
 
+    my $mode;
+    if (-r $path) {$mode = 'r' }else {$mode = '-'}
+    if (-w $path) {$mode .= 'w' }else {$mode .= '-'}
+    if (-x $path) {$mode .= 'x' }else {$mode .= '-'}
+    push @$res, "    $indent<$pref" . "mode code=\"$stat[2]\">$mode</$pref" . 
+      "mode>" 
+      if $details > 1;
+    push @$res, "    $indent<$pref" . "size unit=\"bytes\">$stat[7]</$pref" 
+      . "size>" 
+      if $details > 1;
+    
+    my $atime = localtime($stat[8]);
+    my $mtime = localtime($stat[9]);
+    push @$res, "    $indent<$pref" 
+      . "access-time epoch=\"$stat[8]\">$atime</$pref" . "access-time>"
+      if $details > 2;
+    push @$res, "    $indent<$pref" 
+      . "modify-time epoch=\"$stat[9]\">$mtime</$pref" . "modify-time>"
+      if $details > 1;
+    
+    push @$res, "  $indent</$pref" . "file>" if $details > 1;
 }
 
 sub _get_current_dir {
-my ($res,$details,$i,$dir,$path,$indent,@stat) = @_;    
+    my ($res,$details,$i,$dir,$path,$indent,$ns,@stat) = @_;    
 
-my $line = "$indent<directory name=\"$dir\"";
-$line .= " depth=\"$i\"" if $details > 1;
-$line .= " uid=\"$stat[4]\" gid=\"$stat[5]\"" if $details > 2;
-$line .= '>';
-push @$res, $line;
-push @$res, "  $indent<path>$path</path>" if $details > 1;;
-my $atime = localtime($stat[8]);
-my $mtime = localtime($stat[9]);
-push @$res, "  $indent<access-time epoch=\"$stat[8]\">$atime</access-time>" 
-  if $details > 2;
-push @$res, "  $indent<modify-time epoch=\"$stat[9]\">$mtime</modify-time>" 
-  if $details > 1;
+    my ($pref, $decl) = _ns_data($ns);
+
+    my $line = "$indent<$pref" . "directory name=\"$dir\"";
+    $line .= " depth=\"$i\"" if $details > 1;
+    $line .= " uid=\"$stat[4]\" gid=\"$stat[5]\"" if $details > 2;
+    $line .= '>';
+    push @$res, $line;
+    push @$res, "  $indent<$pref" . "path>$path</$pref" 
+      . "path>" if $details > 1;;
+    my $atime = localtime($stat[8]);
+    my $mtime = localtime($stat[9]);
+    push @$res, "  $indent<$pref" 
+      . "access-time epoch=\"$stat[8]\">$atime</$pref" . "access-time>" 
+      if $details > 2;
+    push @$res, "  $indent<$pref" 
+      . "modify-time epoch=\"$stat[9]\">$mtime</$pref" . "modify-time>" 
+      if $details > 1;
+}
+
+sub _ns_data {
+    my $ns = shift;
+    
+    my $pref = '';
+    my $decl = '';
+    if ($ns->{enabled}) {
+	if ($ns->{prefix}) {
+	    $pref = "$ns->{prefix}:";
+	    $decl = " xmlns:$ns->{prefix}=\"$ns->{uri}\"";
+	} else {
+	    $decl = " xmlns=\"$ns->{uri}\"";
+	}
+    }
+    return ($pref, $decl);
 }
 
 1;
@@ -356,9 +428,7 @@ __END__
 
 =head1 NAME
 
-XML::Directory - Perl extension allowing to get a content of directory 
-including sub-directories as an XML file. You can generate either 
-array/string or SAX events. The current version is 0.51.
+XML::Directory - returns a content of directory as XML
 
 =head1 SYNOPSIS
 
@@ -452,8 +522,13 @@ the constructor. Valid values are 1, 2 or 3.
 
  <?xml version="1.0" encoding="utf-8"?>
  <dirtree>
-   <directory name="test" depth="0">
+   <head version="0.60">
      <path>/home/petr/test</path>
+     <details>2</details>
+     <depth>5</depth>
+   </head>
+   <directory name="test" depth="0">
+     <path></path>
      <modify-time epoch="998300843">Mon Aug 20 11:47:23 2001</modify-time>
      <file name="dir2xml.pl">
        <mode code="33261">rwx</mode>
@@ -469,15 +544,20 @@ the constructor. Valid values are 1, 2 or 3.
 
  <?xml version="1.0" encoding="utf-8"?>
  <dirtree>
-   <directory name="test" depth="0" uid="500" gid="100">
+   <head version="0.60">
      <path>/home/petr/test</path>
-     <access-time epoch="998300915">Mon Aug 20 11:48:35 2001</access-time>
+     <details>3</details>
+     <depth>5</depth>
+   </head>
+   <directory name="test" depth="0" uid="500" gid="100">
+     <path></path>
+     <access-time epoch="999857485">Fri Sep  7 12:11:25 2001</access-time>
      <modify-time epoch="998300843">Mon Aug 20 11:47:23 2001</modify-time>
      <file name="dir2xml.pl" uid="500" gid="100">
        <mode code="33261">rwx</mode>
        <size unit="bytes">225</size>
        <access-time epoch="998300843">Mon Aug 20 11:47:23 2001</access-time>
-       <modify-time epoch="998300843">Mon Aug 20 11:47:23 2001</modify-time>
+        <modify-time epoch="998300843">Mon Aug 20 11:47:23 2001</modify-time>
      </file>
    </directory>
  </dirtree>
@@ -528,7 +608,7 @@ same is true if the path specified can't be found.
  Example:
 
  <?xml version="1.0" encoding="utf-8"?>
- <dirtree>
+ <dirtree xmlns="http://gingerall.org/directory">
  <error number="3">Path /home/petr/work/done not found!</error>
  </dirtree>
 
@@ -593,6 +673,50 @@ tha case of error.
 
 =back
 
+=head2 Methods to deal with namespaces
+
+Generated XML (or SAX events) doesn't contain namespace declarations by
+default (for the sake of simplicity). This feature can enabled manually if
+you want to merge this data with some other XML data, for example.
+
+=over
+
+=item enable_ns;
+
+ $dir->enable_ns; 
+
+Enables the support for namespaces.
+
+=item disable_ns;
+
+ $dir->disable_ns; 
+
+Disables the support for namespaces.
+
+=item get_ns_data;
+
+ $ns  = $dir->get_ns_data;
+
+Returns a hash reference with the following keys:
+
+=over
+
+=item enabled
+
+either 1 or 0
+
+=item uri
+
+namespace URI, 'http://gingerall.org/directory' by default 
+
+=item prefix
+
+namespace prefix, 'xd' by default
+
+=back
+
+=back
+
 =head2 ORIGINAL INTERFACE
 
 =over
@@ -612,6 +736,57 @@ is to keep the get_dir function backward compatible with the version 0.30.
 
 =back
 
+=head2 XML::DIRECTORY::APACHE
+
+This is a mod_perl module that serves as an Apache interface to
+XML::Directory. It allows to send parameters in http request and
+receive a result (XML representation of a directory tree) in http
+response.
+
+Parameters include:
+
+=over
+
+=item path
+
+absolute path to a directory to be parsed, mandatory
+
+The path is not send in query but as an extra path instead. This seems to
+be more appropriate for this kind of parameter.
+
+=item dets
+
+level of details, optional
+
+=item depth
+
+maximal number of nested sub-directories, optional
+
+=item ns
+
+if set to 1, namespaces are used, optional
+
+=back
+
+To use this module, add a similar section to your Apache config file
+
+ <Location /xdir>
+     SetHandler perl-script
+     PerlHandler XML::Directory::Apache
+     PerlSendHeader On
+ </Location>
+
+and send a request to:
+
+ http://hostname/xdir/home/petr/work[?dets=1&depth=1&ns=1]
+
+The path portion following 'xdir' is taken as path; other parameters can
+be send in query.
+
+=head1 VERSION
+
+Current version is 0.60.
+
 =head1 LICENSING
 
 Copyright (c) 2001 Ginger Alliance. All rights reserved. This program is free 
