@@ -23,9 +23,10 @@ sub start_element {
     my ($self, $element) = @_;
     
     print "<$element->{Name} ";
+#   print "xmlns=\"$element->{NamespaceURI}\" ";
     foreach (keys %{$element->{Attributes}}) {
 #   SAX2 way
-    print "$_->{Name}=\"$element->{Attributes}->{$_}->{Value}\" ";
+    print "$element->{Attributes}->{$_}->{Name}=\"$element->{Attributes}->{$_}->{Value}\" ";
 #   SAX1 alternative way
 #   print "$_=\"$element->{Attributes}->{$_}\" ";
     }
